@@ -11,9 +11,12 @@ function(x) {
      if(x == "gpa") return("Generalized Pareto")
      if(x == "gum") return("Gumbel")
      if(x == "kap") return("Kappa")
+     if(x == "kur") return("Kumaraswamy")
+     if(x == "ln3") return("log-Normal3")
      if(x == "nor") return("Normal")
      if(x == "pe3") return("Pearson Type III")
      if(x == "ray") return("Rayleigh")
+     if(x == "rice") return("Rice")
      if(x == "revgum") return("Reverse Gumbel")
      if(x == "wak") return("Wakeby")
      if(x == "wei") return("Weibull")
